library(Rsamtools)
library(tools)


# Load the alignment file and index it with Rsamtools

loadBam <- function(data_path) {
  bamFile <- BamFile(data_path)
  prefix <- file_path_sans_ext(data_path)
  sorted <-
    sortBam(bamFile, paste(prefix, "Rsorted", sep = ""))
  indexBam(sorted)
  return(idxstatsBam(sorted))
}

# Obtain the chromosome ratio

calculateRatio <- function(index) {
  index[1:22, "ratio"] <-
    index[1:22, "mapped"] / (index[1:22, "seqlength"] * 2)
  index[23:24, "ratio"] <-
    index[23:24, "mapped"] /  index[23:24, "seqlength"]
  return(head(index,-1))
}

# Calculate the z-score of ratios from the all chromosomes. Output the z-score
# plot with the index file

calculateZscore <- function(index) {
  index$zscore = (index$ratio - mean(index$ratio)) / sd(index$ratio)
  return(index)
}

calculateCN <- function(index, reference, coverage) {
  reference <- read.table(reference, sep = ',', header = TRUE)
  print(paste("Coverage is", as.double(coverage)))
  index$CN <- index$ratio /  (reference$ratio * as.double(coverage))
  return(index)
}

# Output function for Z-score data. This function will also store .csv file
# in the figures/zscore directory

zscore_output <-
  function(index, out_path, x, y, coverage, scenario) {
    png(out_path, width = x, height = y)
    plot(
      index[, c('seqnames', 'zscore')],
      ylab = "Z-score of ratios",
      xlab = "Chromosomes",
      main = paste("Long reads ", scenario , " with coverage ", coverage),
      las = 2,
      ylim = c(-6, 6)
    )
    dev.off()
    write.csv(index,
              paste(file_path_sans_ext(out_path), ".csv", sep = ""),
              row.names = FALSE)
  }

CN_output <-
  function(index,
           out_path,
           reference,
           x,
           y,
           coverage,
           scenario) {
    png(out_path, width = x, height = y)
    
    plot(
      index[, c('seqnames', 'CN')],
      ylab = "Relative copy number",
      xlab = "Chromosomes",
      main = paste("Long reads ", scenario , " with coverage ", coverage),
      las = 2,
      ylim = c(0, 3)
    )
    dev.off()
  }

bamIndexed <- loadBam(snakemake@input[['long_bam']])
indexWithRatios <- calculateRatio(bamIndexed)
finalIndex <-
  calculateCN(calculateZscore(indexWithRatios),
              snakemake@input[['reference']],
              as.double(snakemake@params[['coverage']]))

print(snakemake@params[['coverage']])
print(as.double(snakemake@params[['coverage']]))

zscore_output(
  finalIndex,
  snakemake@output[['zscore_plot']],
  snakemake@params[['x']],
  snakemake@params[['y']],
  as.double(snakemake@params[['coverage']]),
  snakemake@params[['scenario']]
)

CN_output(
  finalIndex,
  snakemake@output[["copy_number_plot"]],
  snakemake@input[["reference"]],
  snakemake@params[['x']],
  snakemake@params[['y']],
  snakemake@params[['coverage']],
  snakemake@params[['scenario']]
)